@@ -1058,6 +1058,6 @@ extern ulint	btr_cur_n_sea_old;
 extern uint	btr_cur_limit_optimistic_insert_debug;
 #endif /* UNIV_DEBUG */
 
-#include "btr0cur.ic"
+#include "btr0cur.inl"
 
 #endif
