@@ -2,7 +2,7 @@
 
 Copyright (c) 1995, 2017, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2009, Google Inc.
-Copyright (c) 2014, 2019, MariaDB Corporation.
+Copyright (c) 2014, 2022, MariaDB Corporation.
 
 Portions of this file contain modifications contributed and copyrighted by
 Google, Inc. Those modifications are gratefully acknowledged and are described
@@ -83,12 +83,9 @@ reduce the size of the log.
 /** Redo log system */
 log_t	log_sys;
 
-/** Whether to generate and require checksums on the redo log pages */
+/** Whether to require checksums on the redo log pages */
 my_bool	innodb_log_checksums;
 
-/** Pointer to the log checksum calculation function */
-log_checksum_func_t log_checksum_algorithm_ptr;
-
 /* Next log block number to do dummy record filling if no log records written
 for a while */
 static ulint		next_lbn_to_pad = 0;
@@ -674,7 +671,7 @@ log_block_store_checksum(
 /*=====================*/
 	byte*	block)	/*!< in/out: pointer to a log block */
 {
-	log_block_set_checksum(block, log_block_calc_checksum(block));
+	log_block_set_checksum(block, log_block_calc_checksum_crc32(block));
 }
 
 /******************************************************//**
@@ -798,13 +795,9 @@ loop:
 
 /** Flush the recently written changes to the log file.
 and invoke log_mutex_enter(). */
-static
-void
-log_write_flush_to_disk_low()
+static void log_write_flush_to_disk_low()
 {
-	/* FIXME: This is not holding log_sys.mutex while
-	calling os_event_set()! */
-	ut_a(log_sys.n_pending_flushes == 1); /* No other threads here */
+	ut_a(log_sys.n_pending_flushes);
 
 	bool	do_flush = srv_file_flush_method != SRV_O_DSYNC;
 
@@ -812,7 +805,6 @@ log_write_flush_to_disk_low()
 		fil_flush(SRV_LOG_SPACE_FIRST_ID);
 	}
 
-
 	log_mutex_enter();
 	if (do_flush) {
 		log_sys.flushed_to_disk_lsn = log_sys.current_flush_lsn;
@@ -888,7 +880,6 @@ loop:
 #endif
 
 	log_write_mutex_enter();
-	ut_ad(!recv_no_log_write);
 
 	lsn_t	limit_lsn = flush_to_disk
 		? log_sys.flushed_to_disk_lsn
@@ -899,6 +890,8 @@ loop:
 		return;
 	}
 
+	ut_ad(!recv_no_log_write);
+
 	/* If it is a write call we should just go ahead and do it
 	as we checked that write_lsn is not where we'd like it to
 	be. If we have to flush as well then we check if there is a
@@ -1046,12 +1039,101 @@ loop:
 /** write to the log file up to the last log entry.
 @param[in]	sync	whether we want the written log
 also to be flushed to disk. */
-void
-log_buffer_flush_to_disk(
-	bool sync)
+void log_buffer_flush_to_disk(bool sync)
 {
-	ut_ad(!srv_read_only_mode);
-	log_write_up_to(log_get_lsn(), sync);
+  ut_ad(!srv_read_only_mode);
+  log_write_up_to(log_get_lsn(), sync);
+}
+
+
+/** Durably write the log and release log_sys.mutex */
+ATTRIBUTE_COLD void log_write_and_flush()
+{
+  ut_ad(!srv_read_only_mode);
+  ut_ad(!recv_no_log_write);
+  ut_ad(!recv_recovery_is_on());
+
+  /* The following code is adapted from log_write_up_to(). */
+  DBUG_PRINT("ib_log", ("write " LSN_PF " to " LSN_PF,
+                        log_sys.write_lsn, log_sys.lsn));
+  log_sys.n_pending_flushes++;
+  log_sys.current_flush_lsn= log_sys.lsn;
+  os_event_reset(log_sys.flush_event);
+  ut_ad(log_sys.buf_free != log_sys.buf_next_to_write);
+  ulint start_offset= log_sys.buf_next_to_write;
+  ulint end_offset= log_sys.buf_free;
+  ulint area_start= ut_2pow_round(start_offset, ulint(OS_FILE_LOG_BLOCK_SIZE));
+  ulint area_end= ut_calc_align(end_offset, ulint(OS_FILE_LOG_BLOCK_SIZE));
+  ulong write_ahead_size= srv_log_write_ahead_size;
+
+  log_block_set_flush_bit(log_sys.buf + area_start, TRUE);
+  log_block_set_checkpoint_no(log_sys.buf + area_end - OS_FILE_LOG_BLOCK_SIZE,
+                              log_sys.next_checkpoint_no);
+  lsn_t write_lsn= log_sys.lsn;
+  byte *write_buf= log_sys.buf;
+
+  ut_ad(area_end - area_start > 0);
+
+  log_buffer_switch();
+
+  log_sys.log.set_fields(log_sys.write_lsn);
+
+  /* Erase the end of the last log block. */
+  memset(write_buf + end_offset, 0,
+         ~end_offset & (OS_FILE_LOG_BLOCK_SIZE - 1));
+  /* Calculate pad_size if needed. */
+  ulint pad_size= 0;
+  if (write_ahead_size > OS_FILE_LOG_BLOCK_SIZE)
+  {
+    lsn_t end_offset=
+      log_sys.log.calc_lsn_offset(ut_uint64_align_up(write_lsn,
+                                                     OS_FILE_LOG_BLOCK_SIZE));
+    ulint end_offset_in_unit= (ulint) (end_offset % write_ahead_size);
+
+    if (end_offset_in_unit && (area_end - area_start) > end_offset_in_unit)
+    {
+      /* The first block in the unit was initialized after the last
+      writing. Needs to be written padded data once. */
+      pad_size= std::min<ulint>(ulint(write_ahead_size) - end_offset_in_unit,
+                                srv_log_buffer_size - area_end);
+      memset(write_buf + area_end, 0, pad_size);
+    }
+  }
+
+  if (log_sys.is_encrypted())
+    log_crypt(write_buf + area_start, log_sys.write_lsn,
+              area_end - area_start);
+
+  /* Do the write to the log files */
+  log_write_buf(write_buf + area_start, area_end - area_start + pad_size,
+#ifdef UNIV_DEBUG
+                pad_size,
+#endif /* UNIV_DEBUG */
+                ut_uint64_align_down(log_sys.write_lsn,
+                                     OS_FILE_LOG_BLOCK_SIZE),
+                start_offset - area_start);
+  srv_stats.log_padded.add(pad_size);
+  log_sys.write_lsn= write_lsn;
+
+  log_write_mutex_exit();
+
+  /* Code adapted from log_write_flush_to_disk_low() */
+
+  ut_a(log_sys.n_pending_flushes);
+
+  if (srv_file_flush_method != SRV_O_DSYNC)
+    fil_flush(SRV_LOG_SPACE_FIRST_ID);
+
+  log_sys.flushed_to_disk_lsn= log_sys.current_flush_lsn;
+
+  log_sys.n_pending_flushes--;
+
+  os_event_set(log_sys.flush_event);
+
+  const lsn_t flush_lsn= log_sys.flushed_to_disk_lsn;
+  log_mutex_exit();
+
+  innobase_mysql_log_notify(flush_lsn);
 }
 
 /****************************************************************//**
@@ -1366,7 +1448,6 @@ bool log_checkpoint(bool sync)
 
 	log_mutex_enter();
 
-	ut_ad(!recv_no_log_write);
 	oldest_lsn = log_buf_pool_get_oldest_modification();
 
 	/* Because log also contains headers and dummy log records,
@@ -1378,9 +1459,15 @@ bool log_checkpoint(bool sync)
 	flushed up to oldest_lsn. */
 
 	ut_ad(oldest_lsn >= log_sys.last_checkpoint_lsn);
-	if (oldest_lsn
-	    > log_sys.last_checkpoint_lsn + SIZE_OF_MLOG_CHECKPOINT) {
+	const lsn_t age = oldest_lsn - log_sys.last_checkpoint_lsn;
+	if (age > SIZE_OF_MLOG_CHECKPOINT + log_sys.framing_size()) {
 		/* Some log has been written since the previous checkpoint. */
+	} else if (age > SIZE_OF_MLOG_CHECKPOINT
+		   && !((log_sys.log.calc_lsn_offset(oldest_lsn)
+			 ^ log_sys.log.calc_lsn_offset(
+				   log_sys.last_checkpoint_lsn))
+			& ~lsn_t{OS_FILE_LOG_BLOCK_SIZE - 1})) {
+		/* Some log has been written to the same log block. */
 	} else if (srv_shutdown_state > SRV_SHUTDOWN_INITIATED) {
 		/* MariaDB 10.3 startup expects the redo log file to be
 		logically empty (not even containing a MLOG_CHECKPOINT record)
@@ -1392,6 +1479,9 @@ bool log_checkpoint(bool sync)
 		log_mutex_exit();
 		return(true);
 	}
+
+	ut_ad(!recv_no_log_write);
+
 	/* Repeat the MLOG_FILE_NAME records after the checkpoint, in
 	case some log records between the checkpoint and log_sys.lsn
 	need them. Finally, write a MLOG_CHECKPOINT marker. Redo log
