@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1995, 2017, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2017, 2020, MariaDB Corporation.
+Copyright (c) 2017, 2021, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -463,6 +463,89 @@ mtr_t::commit()
   release_resources();
 }
 
+#ifdef UNIV_DEBUG
+/** Check that all pages belong to a shrunk tablespace. */
+struct Shrink
+{
+  const fil_space_t &space;
+  Shrink(const fil_space_t &space) : space(space) {}
+
+  bool operator()(const mtr_memo_slot_t *slot) const
+  {
+    if (!slot->object)
+      return true;
+    switch (slot->type) {
+    default:
+      ut_ad("invalid type" == 0);
+      return false;
+    case MTR_MEMO_MODIFY:
+      break;
+    case MTR_MEMO_SPACE_X_LOCK:
+      ut_ad(&space == slot->object);
+      return true;
+    case MTR_MEMO_PAGE_X_FIX:
+    case MTR_MEMO_PAGE_SX_FIX:
+      const buf_page_t &bpage= static_cast<buf_block_t*>(slot->object)->page;
+      const page_id_t &id= bpage.id;
+      if (id.space() == 0 && id.page_no() == TRX_SYS_PAGE_NO)
+      {
+        ut_ad(srv_is_undo_tablespace(space.id));
+        break;
+      }
+      ut_ad(id.space() == space.id);
+      ut_ad(id.page_no() < space.size);
+      ut_ad(bpage.state == BUF_BLOCK_FILE_PAGE);
+      ut_ad(!bpage.oldest_modification);
+      break;
+    }
+    return true;
+  }
+};
+#endif
+
+/** Commit a mini-transaction that is shrinking a tablespace.
+@param space   tablespace that is being shrunk */
+void mtr_t::commit_shrink(fil_space_t &space)
+{
+  ut_ad(is_active());
+  ut_ad(!is_inside_ibuf());
+  ut_ad(!high_level_read_only);
+  ut_ad(m_modifications);
+  ut_ad(m_made_dirty);
+  ut_ad(!recv_recovery_is_on());
+  ut_ad(m_log_mode == MTR_LOG_ALL);
+  ut_ad(UT_LIST_GET_LEN(space.chain) == 1);
+
+  log_write_and_flush_prepare();
+
+  const lsn_t start_lsn= finish_write(prepare_write());
+
+  log_flush_order_mutex_enter();
+  /* Durably write the reduced FSP_SIZE before truncating the data file. */
+  log_write_and_flush();
+
+  os_file_truncate(space.chain.start->name, space.chain.start->handle,
+                   os_offset_t(space.size) << srv_page_size_shift, true);
+
+  ut_d(m_memo.for_each_block_in_reverse(CIterate<Shrink>(space)));
+
+  m_memo.for_each_block_in_reverse(CIterate<const ReleaseBlocks>
+                                   (ReleaseBlocks(start_lsn, m_commit_lsn,
+                                                  m_flush_observer)));
+  log_flush_order_mutex_exit();
+
+  mutex_enter(&fil_system.mutex);
+  ut_ad(space.is_being_truncated);
+  space.is_being_truncated= false;
+  space.set_stopping(false);
+  mutex_exit(&fil_system.mutex);
+
+  m_memo.for_each_block_in_reverse(CIterate<ReleaseLatches>());
+  srv_stats.log_write_requests.inc();
+
+  release_resources();
+}
+
 /** Commit a mini-transaction that did not modify any pages,
 but generated some redo log on a higher level, such as
 MLOG_FILE_NAME records and a MLOG_CHECKPOINT marker.
