@@ -47,7 +47,7 @@ size_t my_strnxfrmlen_simple(CHARSET_INFO *cs, size_t len)
      is equal to comparing two original strings with my_strnncollsp_xxx().
      
      Not more than 'dstlen' bytes are written into 'dst'.
-     To garantee that the whole string is transformed, 'dstlen' must be
+     To guarantee that the whole string is transformed, 'dstlen' must be
      at least srclen*cs->strnxfrm_multiply bytes long. Otherwise,
      consequent memcmp() may return a non-accurate result.
      
@@ -208,6 +208,18 @@ int my_strnncollsp_simple(CHARSET_INFO * cs, const uchar *a, size_t a_length,
 }
 
 
+static int
+my_strnncollsp_nchars_simple(CHARSET_INFO * cs,
+                             const uchar *a, size_t a_length,
+                             const uchar *b, size_t b_length,
+                             size_t nchars)
+{
+  set_if_smaller(a_length, nchars);
+  set_if_smaller(b_length, nchars);
+  return my_strnncollsp_simple(cs, a, a_length, b, b_length);
+}
+
+
 int my_strnncollsp_simple_nopad(CHARSET_INFO * cs,
                                 const uchar *a, size_t a_length,
                                 const uchar *b, size_t b_length)
@@ -237,7 +249,7 @@ size_t my_casedn_str_8bit(CHARSET_INFO * cs,char *str)
 
 
 size_t my_caseup_8bit(CHARSET_INFO * cs, const char *src, size_t srclen,
-                      char *dst, size_t dstlen)
+                      char *dst, size_t dstlen __attribute__((unused)))
 {
   const char *end= src + srclen;
   register const uchar *map= cs->to_upper;
@@ -249,7 +261,7 @@ size_t my_caseup_8bit(CHARSET_INFO * cs, const char *src, size_t srclen,
 
 
 size_t my_casedn_8bit(CHARSET_INFO * cs, const char *src, size_t srclen,
-                      char *dst, size_t dstlen)
+                      char *dst, size_t dstlen __attribute__((unused)))
 {
   const char *end= src + srclen;
   register const uchar *map=cs->to_lower;
@@ -1535,7 +1547,7 @@ static ulonglong d10[DIGITS_IN_ULONGLONG]=
   Convert a string to unsigned long long integer value
   with rounding.
   
-  SYNOPSYS
+  SYNOPSIS
     my_strntoull10_8bit()
       cs              in      pointer to character set
       str             in      pointer to the string to be converted
@@ -1795,9 +1807,10 @@ ret_sign:
   {
     if (negative)
     {
-      if (ull > (ulonglong) LONGLONG_MIN)
+      if (ull >= (ulonglong) LONGLONG_MIN)
       {
-        *error= MY_ERRNO_ERANGE;
+        if (ull != (ulonglong) LONGLONG_MIN)
+          *error= MY_ERRNO_ERANGE;
         return (ulonglong) LONGLONG_MIN;
       }
       *error= 0;
@@ -2095,6 +2108,7 @@ MY_COLLATION_HANDLER my_collation_8bit_simple_ci_handler =
     my_coll_init_simple,	/* init */
     my_strnncoll_simple,
     my_strnncollsp_simple,
+    my_strnncollsp_nchars_simple,
     my_strnxfrm_simple,
     my_strnxfrmlen_simple,
     my_like_range_simple,
@@ -2111,6 +2125,7 @@ MY_COLLATION_HANDLER my_collation_8bit_simple_nopad_ci_handler =
     my_coll_init_simple,	/* init */
     my_strnncoll_simple,
     my_strnncollsp_simple_nopad,
+    my_strnncollsp_nchars_simple,
     my_strnxfrm_simple_nopad,
     my_strnxfrmlen_simple,
     my_like_range_simple,
